# P112
<?php
class SomeController extends BaseController
{
    public function index($id = null)
    {
        try{
            if (is_null($id) || $id === '') {
                throw new Exception(cmn_msg_param);
            }
            // コントローラのさまざまな処理
            doSomething();
        } catch (Exception $e) {
            log_message($e->getMessage() . '(' .
                $e->getFile() . ':' .
                $e->getLine() . ')');
            $this->tpl->display('error.html',
                ['message' => $e->getMessage()]);
        }
    }
}

# P113
public function __construct($message = null, $code = 0, Exception $previous = null);

# P113
<?php
try {
    throw new Exception('hi');
} catch(Exception $e) {
    print($e->getMessage());
}
// 出力: hi

# P113
<?php
class MyException extends Exception {}

try {
    throw new MyException('hi');
} catch(MyException $e) {
    print('foo');
} catch(Exception $e) {
    // こちらは実行されない
    print('bar');
}
// 出力：foo

# P113
<?php
try {
    f();
} catch(\Exception $e) {
} finally {
    // 必ず呼ばれる
}

# P114
<?php
throw new Exception('a');
// 出力：
//
// Fatal error: Uncaught exception 'Exception' with message 'a' in /private/tmp/a.php:2
// Stack trace:
// #0 {main}
//   thrown in /private/tmp/a.php on line 2

# P114
<?php
set_exception_handler('handler');

// PHP 5.xの場合
function handler(Exception $e)
{
    echo $e->getMessage();
}

// PHP 7.xの場合
function handler(Throwable $e)
{
    echo $e->getMessage();
}

class FooException extends Exception;
class BarException extends Exception;

throw new Exception('test1');

try {
    throw new FooException('test2');
} catch(BarException $e) {
}
// FooExceptionはcatchされていないのでhandlerで捕捉される
// Uncaught Exceptionにはならない
// 出力：
// test1
// test2

# P115
<?php
class SomeController extends BaseController
{
    /**
     * @throws AccessDeniedException
     */
    public function index($id = null)
    {
        if (is_null($id) || $id === '') {
            throw new AccessDeniedException();
        }
        doSomething();
    }
}

# P115
<?php
namespace Fluct\Exception;

class BaseException extends \Exception
    implements Throwable
{
    protected $userMessage;

    /**
     * BaseExceptionのコンストラクタ
     *
     * PHPの規定例外Exceptionを継承し、
     * 第4引数にuserMessageを追加している
     *
     * @param string $message
     * @param int $code
     * @param \Exception|NULL $previous
     * @param string $userMessage
     */
    public function __construct($message = '',
        $code = 0, \Exception $previous = null,
        $userMessage = '')
    {
        parent::__construct($message, $code,
            $previous);
        $this->userMessage = $userMessage;
    }
}

# P116
<?php
// 良くないコードの例
// スタックトレースが引き継がれなくなってしまう
catch(\SomeException $e) {
    throw FluctRuntimeException(
        'some exception occured');
}

# P116
<?php
// 良いコードの例
catch(\SomeException $e) {
    // 例外をFluctRuntimeExceptionで投げなおす
    // スタックトレースを引き継ぐために第3引数で元の例外を渡す
    throw FluctRuntimeException('some', $code, $e);
}

# P116
<?php
class BaseController extends Controller
{
    protected $db;
    protected $user;
    protected $twig;
    protected $auth;

    function __construct()
    {
        parent::__construct();

        $this->db = $this->load->database('sample');
        $this->load->library('Auth');
        $this->user = $this->auth->user_auth($this->db);

        $this->load->library('MyTwig');
        $this->tpl = MyTwig::create();
        $this->tpl->addGlobal('user', $this->user);
    }
}

# P117
<?php
$ci =& get_instance();
$container = $ci->pimple->container;

# P117
<?php
$pimple = new Pimple();

$pimple['ci'] = function() {
    return get_instance();
};

$pimple['db'] = function ($c) {
    return $c['ci']->load->database('sample');
};

$pimple['user'] = function ($c) {
    return $c['auth']->user_auth($c['db']);
};

$pimple['auth'] = function ($c) {
    $c['ci']->load->library('Auth');
    return $c['ci']->auth;
};

$pimple['rawtwig'] = function ($c) {
    $c['ci']->load->library('MyTwig');
    return MyTwig::create();
}

$pimple['twig'] = function ($c) {
    $tpl = $c['rawtwig'];
    $tpl->addGlobal('user', $c['user']);
    return $tpl;
}

# P117
<?php
/**
 * @param Exception $e
 */
function handler(Exception $e)
{
    $ci =& get_instance();
    $container = $ci->pimple->container;
    $twig = $container['twig'];
    log_message('ERR', $e->getMessage());
    if ($e instanceof BaseException) {
        // 例外コードをHTTPレスポンス
        // ステータスコードとして埋める
        header('HTTP', true, $e->getCode());
        $twig->display('error.html',
            $e->getUserMessage());
        return;
    }
    header('HTTP', true, 500);
    $twig->display('error.html',
        $e->getMessage());
}

# P118
$ curl -L http://cs.sensiolabs.org/download/php-cs-fixer-v2.phar -o php-cs-fixer
$ chmod a+x php-cs-fixer
$ ./php-cs-fixer --fix /path/to/project --rules=@PSR2

# P118
<?php
/**
 * @deprecated
 */
function d()
{
    trigger_error(
        'function d() is deprecated since 2017/12/14. use f() instead.',
        E_USER_DEPRECATED);
}
